% Written by Marcin Krzysztofik (Marcin.Krzysztofik@nag.co.uk).  June 2010.

% This function calculates the efficient frontier,
% finds the min and max values of return,
% double checks whether a desired return is within the range.

% Inputs:
%
%   l_c - lower constraint for short selling (0 - no short selling, -1 -
%   short selling up to -100% allowed);
%   
%   u_c - upper constraint for investment (0.5 - up to 50% of the capital 
%   can be invested into each asset);
%
%   portf_ret - the return for which we want to get the standard deviation.
%
% Outputs:
%
%   y - the vector of returns in the frontier;
%   
%   x - the vector of standard deviations in the frontier;
%
%   min_ret - the minimum attainable return in the frontier;
%
%   max_ret - the maximum attainable return in the frontier;
%
%   portf_ret - the adjusted value of return, 
%   which is between min_ret and max_ret.

function [y, x, min_ret, max_ret, portf_ret] = ...
    frontier(l_c, u_c, portf_ret)

% number of points for the plot
num = 50;
x = zeros(num, 1);
y = zeros(num, 1);

% minimum and maximum return, so the optimizer won't try to optimize
% for unattainable returns
[min_ret, max_ret] = get_extreme_returns(l_c, u_c);

% it handles the approximation problem on the boundary
if portf_ret > max_ret
    portf_ret = max_ret;
elseif portf_ret < min_ret
    portf_ret = min_ret;
end

% make a mesh between min_ret and max_ret in x and optimize for each point
for i=0:num-1
    x(i+1) = min_ret+i*(max_ret-min_ret)/(num-1);
    y(i+1) = optimize(x(i+1), l_c, u_c);
end
