% This function demonstrates the use of the the steepest descent (SD)
% algorithm for finding the local minimum of a function - in this case,
% Rosenbrock's Function:
%   F = 100*(x2-x1^2)^2 + (1-x1)^2
%
% It is provided for comparative purposes only: specifically, SD does a
% much poorer job here than the routines in the NAG toolbox (see, e.g.
% e04wd_demo).
%
% Written by Mat Cross (mathewc@nag.co.uk).  August 2007.
% (Slight) modifications by Jeremy.  January 2009.
% - all functions now in same file, where possible.
% - added comments.

% NAG Copyright 2009.

function steepest_descent_demo(pause_time, evals_break)

if nargin<1 || isempty(pause_time)
    pause_time = 0.001;
end

if nargin< 2 || isempty(evals_break)
    evals_break = 400;
end

px = -3 : 0.05 : 1.5;
py = -1 : 0.05 : 3;

[X,Y] = meshgrid(px, py);
pf = 100*(Y - X.^2).^2 + (1 - X).^2;

scrsz = get(0, 'ScreenSize');
figure('Name', 'Minimum of Rosenbrock''s Function using steepest descent', ...
    'Position', [scrsz(1)+100 scrsz(2)+20 scrsz(3)*0.8 scrsz(4)*0.8], ...
    'NumberTitle', 'off');
axes = gca;

% Heights of contours required are in vector v.
v = [5 30 100 220 400 700 1200 2000 3000 4500 6500 9000];
[C,h] = contour(axes, px, py, pf, v, 'Linewidth', 1);
set(h, 'ShowText', 'on', 'TextStep', get(h,'LevelStep')*2);
colormap('prism');
hold on;

% Plot the true minimum with a green circle.
plot(axes, 1, 1, 'go', 'Linewidth', 2, 'MarkerSize', 16);

% Plot the starting point with a blue circle
x = [-2.75 2].';
plot(axes, x(1), x(2), 'bo', 'Linewidth', 2, 'MarkerSize', 16);

title('Steepest descent:  searching for minimum of Rosenbrock''s Function', 'FontSize', 14);

% Add a legend.
frx = [-2.85 -1.15 -1.15 -2.85];
fry = [-0.35 -0.35 -0.775 -0.775];
fill(frx, fry, 'w');
text(-2.75, -0.45, 'f(x,y) = 100(y-x^2)^2 + (1-x)^2', 'FontSize', 12);
plot(axes, -2.75, -0.6, 'bo', 'Linewidth', 2, 'MarkerSize', 10);
text(-2.65, -0.6, 'Starting point', 'FontSize', 10);
plot(axes, -2.75, -0.7, 'go', 'Linewidth', 2, 'MarkerSize', 10);
text(-2.65, -0.7, 'Target minimum', 'FontSize', 10);

% Draw a green line from previous to current point.
line([-2.0,-1.8], [-0.6,-0.6], 'Color', [0.0 1.0 0.0], 'Linewidth', 2);
text(-1.75, -0.6, 'Search path', 'FontSize', 10);

% Create a button and wait for it to be pressed before continuing.
hbutton = uicontrol('Position', [20 20 140 40], ...
    'String', 'Start', 'Callback', @my_button_handler);
uiwait(gcf);

frx = [-0.75 1.4 1.4 -0.75];
fry = [2 2 1.6 1.6];
fill(frx, fry, 'w');

user{1} = [1, pause_time, evals_break];

linesearch(x, @steepest_descent_objfun, 1e-6, user);

set(hbutton, 'String', 'Close');

end

function [x_min, gradnorm, k, xvals] = linesearch(x_0, fhandle, tol, user)
%   Linesearch, choice of search direction and step selection.
%   [x_min, gradnorm, k, xvals]
%        = linesearch(x_0, fhandle, tol, user)
%   performs linesearch for local minimum x_min close to x_0 of function
%   defined by fhandle.m. This M-file must contain information on how to
%   compute grad(f) and Hess(f).
%   Input
%      tol      determines the stopping criterion. Iteration is performed until
%               norm of gradient of f evaluated at the current iterate is
%               <= tol.
%   Output
%      gradnorm norm of grad(f)(x_min)
%      k        number of iterations (not including step selections)
%      xvals    length(x_0)-by-k array, xvals(:,j) is jth approximation
%               to the minimum.
%
%   First 3 input arguments must be specified.
%   Calls steepestdescent and backtrack functions.
%
%   M. Cross, MT6622 Optimization assignment, University of Manchester,
%   2004-5.

if nargin < 3
    error('First 3 arguments MUST be specified.')
end

if tol <= 0
    error('linesearch:nonpositiveTOL', 'TOL must be greater than zero.')
end

% Initialize variables
k = 0;
xvals(:,1) = x_0;
[f, g] = feval(fhandle, x_0, user);
framenumber = 0;

while norm(g) > tol

    % Compute descent direction.
    p = steepestdescent(xvals(:,k+1), fhandle, user);

    % Backtracking linesearch.
    alpha_0 = 1;
    tau = 0.9;
    c_1 = 0.1;
    alpha = backtrack(xvals(:,k+1), fhandle, p, tau, alpha_0, c_1, user);

    % Update x, compute gradient.
    xvals(:,k+2) = xvals(:,k+1) + alpha*p;
    [f, g] = feval(fhandle, xvals(:,k+2), user);

    k = k + 1;

    if user{1}(1) == 1
        % Draw a green line from previous to current point.
        line([xvals(1,k),xvals(1,k+1)], [xvals(2,k),xvals(2,k+1)], 'Color', [0 1 0], 'Linewidth', 2);

        % Display the current number of objective function evaluations.
        frx = [-0.75 1.4 1.4 -0.75];
        fry = [2 2 1.6 1.6];
        fill(frx, fry, 'w');
        t = sprintf('Number of evaluations of objective function = %d', k);
        text(-0.7, 1.9, t, 'FontSize', 12);

        % Set savemovie = 1 to create a bunch of PNG files with a sequence
        % of movie frames.
        savemovie = 0;
        if (savemovie == 1) && ((k < 10) || (mod(k,10) == 0))
            FF = getframe(gcf);
            [FX,Map] = frame2im(FF);
            filename = sprintf('frame%d.png', framenumber);
            framenumber = framenumber + 1;
            imwrite(FX,filename,'png');
        end

        if k == user{1}(3)
            break
        end

        % Slow down the animation.
        pause(user{1}(2));
    end

end

x_min = xvals(:,end);
gradnorm = norm(g);
end

function p = steepestdescent(x, fhandle, user)
%   Normalized steepest descent direction.
%   p = steepestdescent(x, fhandle, user)
%   returns p = -g / ||g||, where g is the
%   gradient of f evaluated at x.

[f, g] = feval(fhandle, x, user);
p = -g/norm(g); % norm(g) ~= 0 if called by linesearch.
end

function [alpha, j] = backtrack(x, fhandle, p, tau, alpha_0, c_1, user)
%   Backtracking linesearch for alpha.
%   [alpha, j] = backtrack(x, fhandle, p, tau, alpha_0, c_1, user)
%   computes the step-size parameter alpha using a backtracking
%   linesearch.
%   Input:
%      p           current search direction
%      tau         reduction parameter
%      alpha_0     initial value for alpha
%      0 < c_1 < 1 constant from the Armijo condition
%   Output:
%      alpha       computed step-size parameter
%      j           number of iterations taken

j = 0;
alpha = alpha_0;
[f, g] = feval(fhandle, x, user);
falpha = feval(fhandle, x + alpha*p, user);

while falpha > f + alpha*c_1*p'*g
    alpha = tau*alpha;
    falpha = feval(fhandle, x + alpha*p, user);
    j = j + 1;
end
end

% The objective function (i.e. the function to be minimized).
function [f, g, G] = steepest_descent_objfun(x, user)
%   Rosenbrock function, its gradient and Hessian.
%   [f, g, G] = rosenbrock(x, user) evaluates
%       f(x) = 100*(x(2) - x(1)^2)^2 + (1 - x(1))^2,
%   g = grad(f)(x) and G = Hess(f)(x).

f = 100*(x(2) - x(1)^2)^2 + (1 - x(1))^2;

if nargout > 1
    g(1) = -400*x(1)*(x(2) - x(1)^2) - 2*(1 - x(1));
    g(2) = 200*(x(2) - x(1)^2);
    g = g';

    if nargout > 2
        G(1,1) = -400*x(2) + 1200*x(1)^2 + 2;
        G(1,2) = -400*x(1);
        G(2,1) = G(1,2);
        G(2,2) = 200;
    end

end
end
