function d02tk_example

% diary d02tk_example.r
fprintf('d02tk example results\n\n');

global omega sqrofr; % For communication with local functions

% Initialize variables and arrays.
neq = 3;
nlbc = 3;
nrbc = 3;
ncol = 7;
mmax = 3;
m = [1; 3; 2];
tols = [1.0e-04; 1.0e-04; 1.0e-04];

% Set values for problem-specific physical parameters.
omega = 1.0;
r = 1.0e+6;

% Set up the mesh.
nmesh = 11;
mxmesh = 51;
ipmesh = zeros(mxmesh, 1);
mesh = zeros(mxmesh, 1);

% Set location of mesh boundaries, then calculate initial spacing.
mesh(1) = 0.0;
mesh(nmesh) = 1.0;
mstep = (mesh(nmesh) - mesh(1))/double(nmesh-1);
for i = 2:nmesh-1
  mesh(i) = mstep*double(i-1);
  ipmesh(i) = 2;
end

% Specify mesh end points as fixed.
ipmesh(1)     = 1;
ipmesh(nmesh) = 1;

% d02tv is a setup routine to be called prior to d02tk.
[work, iwork, ifail] = d02tv( ...
                              nag_int(m), nag_int(nlbc), nag_int(nrbc), ...
                              nag_int(ncol), tols, nag_int(nmesh), mesh, ...
                              nag_int(ipmesh));

% Set number of continuation steps.
ncont = 3;

% We run through the calculation three times with different parameter sets.
for jcont = 1:ncont
  sqrofr = sqrt(r);
  fprintf('\n Tolerance = %8.1e  R = %10.3e\n\n', tols(1), r);

  % Call d02tk to solve BVP for this set of parameters.
  [work, iwork, ifail] = d02tk( ...
                                @ffun, @fjac, @gafun, @gbfun, @gajac, ...
                                @gbjac, @guess, work, iwork);

  % Call d02tz to extract mesh from solution.
  [nmesh, mesh, ipmesh, ermx, iermx, ijermx, ifail] = ...
  d02tz( ...
         nag_int(mxmesh), work, iwork);

  % Output mesh results.
  fprintf(' Used a mesh of %d points\n', nmesh);
  fprintf([' Maximum error = %10.2e in interval %d for component %d\n\n', ...
           ' Mesh points:\n'], ermx, iermx, ijermx);
  for imesh = 1:nmesh
    fprintf( '%4d(%d) %6.4f', imesh, ipmesh(imesh), mesh(imesh));
    if mod(imesh, 4) == 0
      fprintf('\n');
    end
  end

  % Output solution, and store it for plotting.
  xarray = zeros(nmesh, 1);
  yarray = zeros(nmesh, 3);
  fprintf('\n\n    x        f         f''        g\n');
  for imesh = 1:nmesh
    % Call d02ty to perform interpolation on the solution.
    [y, work, ifail] = d02ty( ...
                              mesh(imesh), nag_int(neq), nag_int(mmax), ...
                              work, iwork);
    fprintf(' %6.3f  %8.4f  %8.4f  %8.4f\n', mesh(imesh), ...
            y(1,1), y(2,1), y(3,1));
    xarray(imesh) = mesh(imesh);
    yarray(imesh, 1:3) = y(1:3,1);
  end

  % Plot results for this parameter set.
  if jcont==1
    fig1 = figure;
  elseif jcont==2
    fig2 = figure;
  else
    fig3 = figure;
  end        
  display_plot(xarray, yarray, r)

  % Select mesh for next calculation.
  if jcont < ncont
    r = r*1.0e+02;
    for i = 2:nmesh-1
      ipmesh(i) = 2;
    end

    % d02tx allows the current solution to be used as an initial
    % approximation to the solution of a related problem.
    [work, iwork, ifail] = d02tx( ...
                                  nmesh, mesh, ipmesh, work, iwork);
  end
end

% print(fig1,'-dpng','-r75','d02tk_fig1.png');
% print(fig1,'-deps','-r75','d02tk_fig1.eps');
% print(fig2,'-dpng','-r75','d02tk_fig2.png');
% print(fig2,'-deps','-r75','d02tk_fig2.eps');
% print(fig3,'-dpng','-r75','d02tk_fig3.png');
% print(fig3,'-deps','-r75','d02tk_fig3.eps');

% diary off

function [f] = ffun(x, y, neq, m)
  % Evaluate derivative functions (rhs of system of ODEs).

  global omega sqrofr; % For communication with main routine.
  f = zeros(neq, 1);
  f(1,1) =   y(2,1);
  f(2,1) = -(y(1,1)*y(2,3) + y(3,1)*y(3,2))*sqrofr;
  f(3,1) =  (y(2,1)*y(3,1) - y(1,1)*y(3,2))*sqrofr;

function [dfdy] = fjac(x, y, neq, m)
  % Evaluate Jacobians (partial derivatives of f).

  global omega sqrofr; % For communication with main routine.
  dfdy = zeros(neq, neq, 3);
  dfdy(1,2,1) =  1.0;
  dfdy(2,1,1) = -y(2,3)*sqrofr;
  dfdy(2,2,3) = -y(1,1)*sqrofr;
  dfdy(2,3,1) = -y(3,2)*sqrofr;
  dfdy(2,3,2) = -y(3,1)*sqrofr;
  dfdy(3,1,1) = -y(3,2)*sqrofr;
  dfdy(3,2,1) =  y(3,1)*sqrofr;
  dfdy(3,3,1) =  y(2,1)*sqrofr;
  dfdy(3,3,2) = -y(1,1)*sqrofr;

function [ga] = gafun(ya, neq, m, nlbc)
  % Evaluate boundary conditions at left-hand end of range.

  global omega sqrofr; % For communication with main routine.
  ga = zeros(nlbc, 1);
  ga(1) = ya(1);
  ga(2) = ya(2);
  ga(3) = ya(3) - omega;

function [dgady] = gajac(ya, neq, m, nlbc)
  % Evaluate Jacobians (partial derivatives of ga).

  dgady = zeros(nlbc, neq, 3);
  dgady(1,1,1) = 1.0;
  dgady(2,2,1) = 1.0;
  dgady(3,3,1) = 1.0;

function [gb] = gbfun(yb, neq, m, nrbc)
  % Evaluate boundary conditions at right-hand end of range.

  global omega sqrofr; % For communication with main routine.
  gb = zeros(nrbc, 1);
  gb(1) = yb(1);
  gb(2) = yb(2);
  gb(3) = yb(3) + omega;

  function [dgbdy] = gbjac(yb, neq, m, nrbc)
    % Evaluate Jacobians (partial derivatives of gb).

    dgbdy = zeros(nrbc, neq, 3);
    dgbdy(1,1,1) = 1.0;
    dgbdy(2,2,1) = 1.0;
    dgbdy(3,3,1) = 1.0;

function [y, dym] = guess(x, neq, m)
  % Evaluate initial approximations to solution components and derivatives.

  global omega sqrofr; % For communication with main routine.
  y = zeros(neq, 3);
  dym = zeros(neq, 1);
  y(1,1) = -x^2*(x - 0.5)*(x - 1.0)^2;
  y(2,1) = -x*(x - 1.0)*(5.0*x^2 - 5.0*x + 1.0);
  y(3,1) = -8.0*omega*(x - 0.5)^3;
  y(2,2) = -(20.0*x^3 - 30.0*x^2 + 12.0*x - 1.0);
  y(2,3) = -(60.0*x^2 - 60.0*x + 12.0*x);
  y(3,2) = -24.0*omega*(x - 0.5)^2;

  dym(1) = y(2,1);
  dym(2) = -(120.0*x - 60.0);
  dym(3) = -56.0*omega*(x - 0.5);

function display_plot(x, y, r)
  % Plot two of the curves, then add the other one.
  [haxes, hline1, hline2] = plotyy(x, y(:,2), x, y(:,3));
  % We want the third curve to be plotted on the left-hand y-axis.
  hold(haxes(1), 'on');
  hline3 = plot(x, y(:,1));
  % Set the axis limits and the tick specifications to beautify the plot.
  set(haxes(1), 'YLim', [-0.1 0.4]);
  set(haxes(1), 'XMinorTick', 'on', 'YMinorTick', 'on');
  set(haxes(1), 'YTick', [-0.1:0.1:0.4]);
  set(haxes(2), 'YLim', [-1 1]);
  set(haxes(2), 'YMinorTick', 'on');
  set(haxes(2), 'YTick', [-1:0.5:1]);
  for iaxis = 1:2
    % These properties must be the same for both sets of axes.
    set(haxes(iaxis), 'XLim', [0 1]);
    set(haxes(iaxis), 'XTick', [0:0.2:1]);
  end
  set(gca, 'box', 'off'); % no ticks on opposite axes.
  % Add title.
  ord = log10(r);
  title({'Incompressible Fluid Flow between Discs.', ...
         ['Solutions for Re = 10^n, n = ', num2str(ord)]});
  % Label the axes.
  xlabel('x');
  ylabel(haxes(1), 'f and f''');
  ylabel(haxes(2), 'g');
  % Add a legend.
  legend('f''','f','g','Location','Best')
  % Set some features of the three lines.
  set(hline1, 'Linewidth', 0.25, 'Marker', '+', 'LineStyle', '-', ...
      'Color', 'Magenta');
  set(hline2, 'Linewidth', 0.25, 'Marker', 'x', 'LineStyle', '--');
  set(hline3, 'Linewidth', 0.25, 'Marker', '*', 'LineStyle', ':');
