function d02tl_example

% diary d02tl_example.r
fprintf('d02tl example results\n\n');

global omega sqrofr

% Initialize variables and arrays.
neq  = nag_int(3);
nlbc = neq;
nrbc = neq;
ncol = nag_int(7);
mmax = nag_int(3);
m    = nag_int([1; 3; 2]);
tols = [1; 1; 1]/10^4;

% Set values for problem-specific physical parameters.
omega = 1;
r     = 10^6;

% Set up the mesh.
nmesh  = nag_int(11);
mxmesh = nag_int(51);
ipmesh = zeros(mxmesh, 1, nag_int_name);
mesh   = zeros(mxmesh, 1);

mesh(1:nmesh)     = [0:0.1:1];
ipmesh(1)         = 1;
ipmesh(2:nmesh-1) = 2;
ipmesh(nmesh)     = 1;

% d02tv is a setup routine to be called prior to d02tl.
[rcomm, icomm, ifail] = d02tv(...
                              m, nlbc, nrbc, ncol, tols, nmesh, mesh, ipmesh);

% Set number of continuation steps.
ncont = 3;

% We run through the calculation three times with different parameter sets.
for jcont = 1:ncont
  sqrofr = sqrt(r);
  fprintf('\n Tolerance = %8.1e  R = %10.3e\n\n', tols(1), r);

  % Call d02tk to solve BVP for this set of parameters.
  [rcomm, icomm, ifail] = ...
    d02tl(...
          @ffun, @fjac, @gafun, @gbfun, @gajac, @gbjac, @guess, rcomm, icomm);

  % Call d02tz to extract mesh from solution.
  [nmesh, mesh, ipmesh, ermx, iermx, ijermx, ifail] = ...
    d02tz( ...
           mxmesh, rcomm, icomm);

  % Output mesh results.
  fprintf(' Mesh size = %d\n', nmesh);
  fprintf(' Maximum error = %10.2e in interval %d for component %d\n\n', ...
          ermx, iermx, ijermx);
  fprintf('Mesh points:\n');
  for imesh = 1:nmesh
    fprintf( '%4d(%d) %6.4f', imesh, ipmesh(imesh), mesh(imesh));
    if mod(imesh, 4) == 0
      fprintf('\n');
    end
  end

  % Output solution, and store it for plotting.
  xarray = zeros(nmesh, 1);
  yarray = zeros(nmesh, 3);
  fprintf('\n\n    x        f         f''        g\n');
  for imesh = 1:nmesh
    % Call d02ty to perform interpolation on the solution.
    [y, rcomm, ifail] = d02ty(...
                              mesh(imesh), neq, mmax, rcomm, icomm);
    fprintf(' %6.3f  %8.4f  %8.4f  %8.4f\n', mesh(imesh), y(:,1));
    xarray(imesh) = mesh(imesh);
    yarray(imesh, 1:3) = y(1:3,1);
  end

  % Plot results for this parameter set.
  if jcont==1
    fig1 = figure;
  else
    if jcont==2
      fig2 = figure;
    else
      fig3 = figure;
    end
  end
  display_plot(xarray, yarray, r);

  % Select mesh for next calculation.
  if jcont < ncont
    r = 100*r;
    ipmesh(2:nmesh-1) = 2;

    % d02tx allows the current solution to be used for continuation
    [rcomm, icomm, ifail] = d02tx(...
                                  nmesh, mesh, ipmesh, rcomm, icomm);
  end
end
% print(fig1,'-dpng','-r75','d02tl_fig1.png');
% print(fig1,'-deps','-r75','d02tl_fig1.eps');
% print(fig2,'-dpng','-r75','d02tl_fig2.png');
% print(fig2,'-deps','-r75','d02tl_fig2.eps');
% print(fig3,'-dpng','-r75','d02tl_fig3.png');
% print(fig3,'-deps','-r75','d02tl_fig3.eps');

% diary off

function [f, user] = ffun(x, y, neq, m, user)
  % Evaluate derivative functions (rhs of system of ODEs).

  global omega sqrofr; % For communication with main routine.
  f = zeros(neq, 1);
  f(1,1) =   y(2,1);
  f(2,1) = -(y(1,1)*y(2,3) + y(3,1)*y(3,2))*sqrofr;
  f(3,1) =  (y(2,1)*y(3,1) - y(1,1)*y(3,2))*sqrofr;

function [dfdy, user] = fjac(x, y, neq, m, dfdy, user)
  % Evaluate Jacobians (partial derivatives of f).

  global omega sqrofr
  dfdy = zeros(neq, neq, 3);
  dfdy(1,2,1) =  1.0;
  dfdy(2,1,1) = -y(2,3)*sqrofr;
  dfdy(2,2,3) = -y(1,1)*sqrofr;
  dfdy(2,3,1) = -y(3,2)*sqrofr;
  dfdy(2,3,2) = -y(3,1)*sqrofr;
  dfdy(3,1,1) = -y(3,2)*sqrofr;
  dfdy(3,2,1) =  y(3,1)*sqrofr;
  dfdy(3,3,1) =  y(2,1)*sqrofr;
  dfdy(3,3,2) = -y(1,1)*sqrofr;

function [ga, user] = gafun(ya, neq, m, nlbc, user)
  % Evaluate boundary conditions at left-hand end of range.

  global omega sqrofr
  ga = zeros(nlbc, 1);
  ga(1) = ya(1);
  ga(2) = ya(2);
  ga(3) = ya(3) - omega;

function [dgady, user] = gajac(ya, neq, m, nlbc, dgady, user)
  % Evaluate Jacobians (partial derivatives of ga).

  dgady = zeros(nlbc, neq, 3);
  dgady(1,1,1) = 1;
  dgady(2,2,1) = 1;
  dgady(3,3,1) = 1;

function [gb, user] = gbfun(yb, neq, m, nrbc, user)
  % Evaluate boundary conditions at right-hand end of range.

  global omega sqrofr
  gb = zeros(nrbc, 1);
  gb(1) = yb(1);
  gb(2) = yb(2);
  gb(3) = yb(3) + omega;

function [dgbdy, user] = gbjac(yb, neq, m, nrbc, dgbdy, user)
  % Evaluate Jacobians (partial derivatives of gb).

  dgbdy = zeros(nrbc, neq, 3);
  dgbdy(1,1,1) = 1;
  dgbdy(2,2,1) = 1;
  dgbdy(3,3,1) = 1;

function [y, dym, user] = guess(x, neq, m, user)
  % Evaluate initial approximations to solution components and derivatives.

  global omega sqrofr
  y = zeros(neq, 3);
  dym = zeros(neq, 1);
  y(1,1) = -x^2*(x - 0.5)*(x - 1)^2;
  y(2,1) = -x*(x - 1)*(5*x^2 - 5*x + 1);
  y(3,1) = -8*omega*(x - 0.5)^3;
  y(2,2) = -(20*x^3 - 30*x^2 + 12*x - 1);
  y(2,3) = -(60*x^2 - 60*x + 12*x);
  y(3,2) = -24*omega*(x - 0.5)^2;

  dym(1) = y(2,1);
  dym(2) = -(120*x - 60);
  dym(3) = -56*omega*(x - 0.5);

function display_plot(x, y, r)
  % Plot two of the curves, then add the other one.
  [haxes, hline1, hline2] = plotyy(x, y(:,2), x, y(:,3));
  % We want the third curve to be plotted on the left-hand y-axis.
  hold(haxes(1), 'on');
  hline3 = plot(x, y(:,1));
  % Set the axis limits and the tick specifications to beautify the plot.
  set(haxes(1), 'YLim', [-0.1 0.4]);
  set(haxes(2), 'YLim', [-1 1]);
  set(haxes(1), 'XMinorTick', 'on', 'YMinorTick', 'on');
  set(haxes(2), 'YMinorTick', 'on');
  set(haxes(1), 'YTick', [-0.1:0.1:0.4]);
  set(haxes(2), 'YTick', [-1:0.5:1]);
  for iaxis = 1:2
    % These properties must be the same for both sets of axes.
    set(haxes(iaxis), 'XLim', [0 1]);
    set(haxes(iaxis), 'XTick', [0:0.2:1]);
  end
  set(gca, 'box', 'off');
  % Add title.
  ord = log10(r);
  title(['Flow between Discs, Re = 10^n, n = ', num2str(ord)]);
  % Label the axes.
  xlabel('x');
  ylabel(haxes(1), 'f and f''');
  ylabel(haxes(2), 'g');
  % Add a legend.
  legend('f''','f','g','Location','Best')
  % Set some features of the three lines.
  set(hline1, 'Linewidth', 0.25, 'Marker', '+', 'LineStyle', '-', ...
     'Color', 'Magenta');
  set(hline2, 'Linewidth', 0.25, 'Marker', 'x', 'LineStyle', '--');
  set(hline3, 'Linewidth', 0.25, 'Marker', '*', 'LineStyle', ':');
