function nag_ode_bvp_coll_nlin_setup_example

% diary nag_ode_bvp_coll_nlin_setup_example.r
fprintf('nag_ode_bvp_coll_nlin_setup example results\n\n');

% Global constants communicated to local functions ffun and fjac
global  beta0 eta lambda mu
% Set values for problem-specific physical parameters.
eta    = 0.01;
mu     = 0.02;
lambda = 0.0279;
beta0  = 1575.0;

% Initialize variables and arrays.
neq  = nag_int(6);
nlbc = nag_int(3);
nrbc = nlbc;
ncol = nag_int(5);
mmax = nag_int(1);

% Set up the mesh.
nmesh  = nag_int(7);
mxmesh = nag_int(8*nmesh);
ipmesh = zeros(mxmesh, 1, nag_int_name);
mesh   = zeros(mxmesh, 1);

% Set location of mesh boundaries, then calculate initial spacing.
mstep           = 1/double(nmesh-1);
mesh(1:nmesh)   = 0:mstep:1;
ipmesh(1:nmesh) = nag_int(2);

% Specify mesh end points as fixed.
ipmesh(1)     = nag_int(1);
ipmesh(nmesh) = nag_int(1);

m(1:neq)      = nag_int(1);
tols(1:neq)   = 1e-6;

% nag_ode_bvp_coll_nlin_setup is a setup routine to be called prior to nag_ode_bvp_coll_nlin.
[work, iwork, ifail] = nag_ode_bvp_coll_nlin_setup(...
                             m, nlbc, nrbc, ncol, tols, nmesh, mesh, ipmesh);

% Call nag_ode_bvp_coll_nlin to solve BVP for this set of parameters.
[work, iwork, ifail] = nag_ode_bvp_coll_nlin(...
                             @ffun, @fjac, @gafun, @gbfun, @gajac, @gbjac,...
                             @guess, work, iwork);

% Call nag_ode_bvp_coll_nlin_diag to extract mesh from solution.
[nmesh, mesh, ipmesh, ermx, iermx, ijermx, ifail] = ...
nag_ode_bvp_coll_nlin_diag(...
       mxmesh, work, iwork);

% Output mesh results.
fprintf(' Used a mesh of %d points\n', nmesh);
fprintf(' Maximum error = %10.2e in interval %d for component %d\n',...
        ermx, iermx, ijermx);

% Output solution, and store it for plotting.
yarray           = zeros(nmesh, 3);
xarray(1:nmesh)  = mesh(1:nmesh);
for i = 1:nmesh
  % Call nag_ode_bvp_coll_nlin_interp to perform interpolation on the solution.
  [y, work, ifail] = nag_ode_bvp_coll_nlin_interp(mesh(i), neq, mmax, work, iwork);
  yarray(i, 1:3)   = y(1:3,1);
end

% Plot results.
fig1 = figure;
display_plot(xarray, yarray);
% print(fig1,'-dpng','-r75','nag_ode_bvp_coll_nlin_setup_fig1.png')
% print(fig1,'-deps','-r75','nag_ode_bvp_coll_nlin_setup_fig1.eps');

% diary off

function [f] = ffun(x, y, neq, m)
  % Evaluate derivative functions (rhs of system of ODEs).
  global beta0 eta lambda mu

  beta = beta0*(1 + cos(2*pi*x));
  f    = zeros(neq, 1);
  f(1) = mu - beta*y(1,1)*y(3,1);
  f(2) = beta*y(1,1)*y(3,1) - y(2,1)/lambda;
  f(3) = y(2,1)/lambda - y(3,1)/eta;

function [dfdy] = fjac(x, y, neq, m)
  % Evaluate Jacobians (partial derivatives of f).
  global beta0 eta lambda mu

  beta = beta0*(1 + cos(2*pi*x));
  dfdy = zeros(neq, neq, 1);
  dfdy(1,1,1) = -beta*y(3,1);
  dfdy(1,3,1) = -beta*y(1,1);
  dfdy(2,1,1) =  beta*y(3,1);
  dfdy(2,2,1) = -1/lambda;
  dfdy(2,3,1) =  beta*y(1,1);
  dfdy(3,2,1) =  1/lambda;
  dfdy(3,3,1) = -1/eta;

function [ga] = gafun(ya, neq, m, nlbc)
  % Evaluate boundary conditions at left-hand end of range.

  ga    = zeros(nlbc, 1);
  ga(1) = ya(1) - ya(4);
  ga(2) = ya(2) - ya(5);
  ga(3) = ya(3) - ya(6);
  
function [dgady] = gajac(ya, neq, m, nlbc)
  % Evaluate Jacobians (partial derivatives of ga).

  dgady        = zeros(nlbc, neq, 1);
  dgady(1,1,1) =  1;
  dgady(1,4,1) = -1;
  dgady(2,2,1) =  1;
  dgady(2,5,1) = -1;
  dgady(3,3,1) =  1;
  dgady(3,6,1) = -1;

function [gb] = gbfun(yb, neq, m, nrbc)
  % Evaluate boundary conditions at right-hand end of range.

  gb    = zeros(nrbc, 1);
  gb(1) = yb(1) - yb(4);
  gb(2) = yb(2) - yb(5);
  gb(3) = yb(3) - yb(6);

function [dgbdy] = gbjac(yb, neq, m, nrbc)
  % Evaluate Jacobians (partial derivatives of gb).

  dgbdy        = zeros(nrbc, neq, 1);
  dgbdy(1,1,1) =  1;
  dgbdy(1,4,1) = -1;
  dgbdy(2,2,1) =  1;
  dgbdy(2,5,1) = -1;
  dgbdy(3,3,1) =  1;
  dgbdy(3,6,1) = -1;

function [y, dym] = guess(x, neq, m)
  % Evaluate initial approximations to solution components and derivatives.

  y      = zeros(neq, 1);
  dym    = zeros(neq, 1);
  y(1:3) = 1;
  y(4:6) = y(1:3);

function display_plot(x, y)
  % Plot two of the curves, then add the other one.
  [haxes, hline1, hline2] = plotyy(x, y(:,1), x, y(:,2), 'plot', 'semilogy');
  % Third curve to be plotted on the right-hand y-axis.
  axes(haxes(2));
  hold on
  hline3 = plot(x, y(:,3));
  % Set the axis limits and the tick specifications to beautify the plot.
  set(haxes(1), 'YLim', [0.06 0.08]);
  set(haxes(2), 'YLim', [1e-10 1]);
  set(haxes(1), 'XMinorTick', 'on', 'YMinorTick', 'on');
  set(haxes(2), 'YMinorTick', 'on');
  set(haxes(1), 'YTick', [0.06  0.065 0.07 0.075 0.08]);
  set(haxes(2), 'YTick', [1e-10 1e-8  1e-6 1e-4  1e-2  1]);
  for iaxis = 1:2
    % These properties must be the same for both sets of axes.
    set(haxes(iaxis), 'XLim',  [0 1]);
    set(haxes(iaxis), 'XTick', [0:0.2:1]);
  end
  set(gca, 'box', 'off');
  % Add title.
  title('Model of Spread of Measles');
  % Label the axes.
  xlabel('Time');
  ylabel(haxes(1), 'Susceptibles');
  ylabel(haxes(2), 'Infectives and Latents');
  % Add a legend.
  legend('Infectives','Latents','Susceptibles','Location','NorthEast')
  % Set some features of the three lines.
  set(hline1, 'Linewidth', 0.25, 'Marker', '+', 'LineStyle', '-');
  set(hline2, 'Linewidth', 0.25, 'Marker', 'x', 'LineStyle', '--', ...
      'Color', 'Magenta');
  set(hline3, 'Linewidth', 0.25, 'Marker', '*', 'LineStyle', ':');
