function nag_ode_bvp_shoot_genpar_intern_example

% diary nag_ode_bvp_shoot_genpar_intern_example.r
fprintf('nag_ode_bvp_shoot_genpar_intern example results\n\n');

% For communication with prsol.
global iprint;

% Set up initial values before calling NAG routine for the first time.
h = 0.1;
e = [0.0001; 0.0001];
parerr = [1e-05; 0.001];
param = [0.2; 0];
m1 = 6;
n = 2;
marray = zeros(1,m1);

% First example - parameterized two-point boundary-value problem.
fprintf('Case 1 \n\n');

% Set iprint to 1 to obtain output from prsol at each iteration.
iprint = 0;

[h, param, c, ifail] = ...
nag_ode_bvp_shoot_genpar_intern(...
       h, e, parerr, param, nag_int(m1), @aux1, @bcaux1, @raaux1, @prsol);

% Output results.
fprintf('Final parameters \n');
disp(sprintf('   %10.2e',param));
fprintf('\n Final solution \n');
fprintf('   X-value      Components of solution\n');

% Call the aux routine to evaluate the derivatives.
[x0, x1, r] = raaux1(param);

% h is steplength for which derivatives were evaluated.
h = (x1-x0)/double(m1-1);
for i = 1:m1;
  m = x0 + double(i-1)*h;
  fprintf('%8.2f    ',m);
  fprintf('%14.4e',c(i,1:n));
  fprintf('\n');
  % Store the m values so they can be plotted.
  marray(i) = m;
end

% Second example - find gravitational constant and range given
% projectile terminal velocity.
fprintf('\n\n Case 2 \n\n');

% Set iprint to 1 to obtain output from prsol at each iteration.
iprint = 0;

h = 10.0;
param = [32.0; 6000.0; 0.54];
parerr = [1.0e-5; 1.0e-4; 1.0e-4];
e = [1.0e-2; 1.0e-2; 1.0e-2];
n = 3;
m1 = 6;
qarray = zeros(1,m1);

[h, param, c1, ifail] = ...
nag_ode_bvp_shoot_genpar_intern( ...
       h, e, parerr, param, nag_int(m1), @aux2, @bcaux2, @raaux2, @prsol);

% Output results.
fprintf('Final parameters\n');
fprintf('   %10.2e',param);
fprintf('\n\nFinal solution\n');
fprintf('   X-value      Components of solution\n');

% Call the aux routine to evaluate the derivatives.
[x0, x1, r] = raaux2(param);

% h is steplength for which derivatives were evaluated.
h = (x1-x0)/double(m1-1);
for i = 1:m1;
  q = x0 + double(i-1)*h;
  fprintf('%8.0f    ',q);
  fprintf('%14.4e',c1(i,1:n));
  fprintf('\n');
  % Store the q values so they can be used for plotting.
  qarray(i) = q;
end

% Plot results.
fig1 = figure;
display_plot(marray, c, 'Solution and Derivative');
fig2 = figure;
display_plot(qarray, c1, 'Height and Velocity');
% print(fig1,'-dpng','-r75','nag_ode_bvp_shoot_genpar_intern_fig1.png');
% print(fig1,'-deps','-r75','nag_ode_bvp_shoot_genpar_intern_fig1.eps');
% print(fig2,'-dpng','-r75','nag_ode_bvp_shoot_genpar_intern_fig2.png');
% print(fig2,'-deps','-r75','nag_ode_bvp_shoot_genpar_intern_fig2.eps');

% diary off

function [f] = aux1(y,x,param)
  % Evaluate the derivatives (case 1).
  f(1) = y(2);
  f(2) = (y(1)^3-y(2))/(2.0*x);

function [g0,g1] = bcaux1(param)
  % Evaluate the derivatives at the end-points (case 1).
  z = 0.1;
  g0(1) = 0.1 + param(1)*sqrt(z)*0.1 + 0.01*z;
  g0(2) = param(1)*0.05/sqrt(z) + 0.01;
  g1(1) = 1.0/6.0;
  g1(2) = param(2);

function [] = prsol(param, resid, n, err)
  % For communication with main routine.
  global iprint;
  if iprint == 1
    fprintf('Current parameters: ');
    fprintf('%15.6e ', param(1:n));
    fprintf('\nResiduals: ');
    fprintf('%13.6e ', err(1:n));
    fprintf('\nSum of residuals squared = %13.6e\n', resid);
  end

function [x0, x1, r] = raaux1(param)
  % Evaluate the end-points (case 1).
  x0 = 0.1;
  x1 = 16;
  r  = 16;

function [f] = aux2(y,x,param)
  % Evaluate the derivatives (case 2).
  c = cos(y(3));
  s = sin(y(3));
  f(1) = s/c;
  f(2) = -(param(1)*s+0.00002*y(2)*y(2))/(y(2)*c);
  f(3) = -param(1)/(y(2)*y(2));
  
function [g0,g1] = bcaux2(param)
  % Evaluate the derivatives at the end-points (case 2).
  g0(1) = 0;
  g0(2) = 500;
  g0(3) = 0.5;
  g1(1) = 0;
  g1(2) = 450;
  g1(3) = param(3);

function [x0, x1, r] = raaux2(param)
  % Evaluate the end-points (case 2).
  x0 = double(0);
  x1 = param(2);
  r  = param(2);

function display_plot(data, index, ylabelString)
  % Choose the type of plot based on y axis label.
  if strncmp(ylabelString, 'Solution', 8)
    % Plot the results as a linear graph.
    plot(data,index(:,1),'-+',data,index(:,2),'--x');
    % Set the axis limits and the tick specifications to beautify the plot.
    set(gca, 'XLim', [0 16]);
    set(gca, 'XTick', [0 4 8 12 16]);
    % Set the title.
    title('Parameterized Two-point Boundary-value Problem');
    % Label the axes.
    xlabel('x');
    ylabel(ylabelString);
    % Add a legend.
    legend('solution y(x)','derivative y''(x)','Location','Best');
  else
    % Plot the results as a linear graph (2 y axes).
    [haxes, hline1, hline2] = plotyy(data,index(:,1),data,index(:,3));
    hold on
    % Add a third curve.
    hline3 = plot(data,index(:,2));
    % Set the axis limits and the tick specifications to beautify the plot.
    set(haxes(1), 'YLim', [0 850]);
    set(haxes(1), 'YMinorTick', 'on');
    set(haxes(1), 'YTick', [0 200 400 600 800]);
    set(haxes(2), 'YLim', [-1 1]);
    set(haxes(2), 'YMinorTick', 'on');
    set(haxes(2), 'YTick', [-1 -0.5 0 0.5 1]);
    for iaxis = 1:2
        % These properties must be the same for both sets of axes.
        set(haxes(iaxis), 'XLim', [0 6000]);
        set(haxes(iaxis), 'XTick', [0 1000 2000 3000 4000 5000 6000]);
    end
    set(gca, 'box', 'off'); % no ticks on opposite axes.
    % Set the title.
    title(['Range from Projectile Terminal Velocity']);
    % Label the axes.
    xlabel('x');
    ylabel(haxes(1),ylabelString);
    ylabel(haxes(2),'Angle');
    % Add a legend.
    legend('angle','height','velocity','Location','Best');
    % Set some features of the three lines.
    set(hline1, 'Linewidth', 0.5, 'Marker', '+', 'LineStyle', '-');
    set(hline2, 'Linewidth', 0.5, 'Marker', '*', 'LineStyle', '-');
    set(hline3, 'Linewidth', 0.5, 'Marker', 'x', 'LineStyle', '--', ...
        'Color', 'Magenta');
  end
