function res = fem_solve(handles)
    clc
    
    % define domains and constraints
    domains = handles.domains;
    conditions = handles.constraints;
    

    % create solution structure
    handles.fem.solutions = struct('result',{},'time',{});
    
    % time step
    dt=handles.fem.time.dt;
    % end time 
    T=handles.fem.time.T;
    
    % for each domain (can only be one at the moment)
    for k=1:length(domains)

            % current domain
            curDomain = domains(k);
            % connectivity of the mesh
            conn = curDomain.mesh.conn;
            % coordinates of the mesh nodes
            coor = curDomain.mesh.coor;
            
            
            edges = [];
            for z=1:size(curDomain.boundaries,2)
                % generates a single matrix describing how each polygon edge joins
                % onto the next for the whole domain rather than just one
                % of the boundaries
                nedges = curDomain.boundaries(z).edge;
                nedges(1:2,:) = nedges(1:2,:)+size(edges,2);
                % the final matrix
                edges = [edges, nedges ];
            end

            
            if isempty(curDomain.boundary)
                fprintf('[ FAIL ]\n');
                continue;
            end
            
            % the edge matrix for the boundary mesh of the whole domain
            gedges = curDomain.boundary.edge;   

            ggedges = [];
            for i=1:size(gedges,2)
                % puts a fourth row in ggedges containing the tag of the
                % constraint assigned to that edge
                ggedges(:,i) = [gedges(:,i);edges(3,gedges(3,i))];
            end
            
            Z = zeros(1,size(ggedges,2));
            % finds all the constrained edges
            constrEdges = ggedges(:, ggedges(4,:)~=Z);
            cE = size(constrEdges,2);
            
            % differentiate here which of these diricEdges are dirichlet
            % and which are neumann conditions
            
            % preallocate logical array for seperating
            DiricNeumann = zeros(1,cE);
            for i = 1:size(conditions,2)
                LogArray = i*ones(1,cE);
                conditionType = (conditions(i).type - 1)*ones(1,cE);
                LogArray = constrEdges(4,:) == LogArray;
                DiricNeumann(LogArray) = conditionType(LogArray);
            end
            DiricNeumann = logical(DiricNeumann);
            
            diricEdges = constrEdges(:,~DiricNeumann);
            neumannEdges = constrEdges(:,DiricNeumann);
            
            
            % determines which vertices have dirichlet boundary conditions on them
            [diricVertices,m] = unique([diricEdges(1,:), diricEdges(2,:)]);
            % pulls out the tag of which constraint is on each edge and puts
            % it into types
            ty = [diricEdges(4,:),diricEdges(4,:)];
            types = ty(m);
            
             % determines which vertices have neumann boundary conditions on them
            [neumannVertices,M] = unique([neumannEdges(1,:), neumannEdges(2,:)]);
            % pulls out the tag of which constraint is on each edge and puts
            % it into types
            nty = [neumannEdges(4,:),neumannEdges(4,:)];
            ntypes = nty(M);



            % indices of unconstrained vertices of the mesh
            freeVertices = setdiff(1:size(coor,2), [diricVertices] );  
            
            % number of vertices in the mesh
            ndf = size(coor,2);
               
            % initialise the stiff and mass matrices
            stiff = sparse(ndf,ndf);
            mass = sparse(ndf,ndf);
            
            
            U = zeros(ndf, ceil(T/dt)+1 );
    
            % matrix assembly
            for l=1:size(curDomain.mesh.conn,2)
                % for each triangle in the mesh work out where the center
                % is
                 c = coor(:,conn(:,l))';
                 center = sum(c)/3;
                
                 % triangle area
                 Ar = abs((c(2,1)*c(1,2)-c(1,1)*c(2,2))+(c(3,1)*c(2,2)-c(2,1)*c(3,2))+(c(1,1)*c(3,2)-c(3,1)*c(1,2)))/2;
                 M = [ones(3,1),c];
                 g = inv(M);
                 G = g(2:3,:)'*g(2:3,:);
                 fv = fFn(center(1), center(2), 0,handles );
                
                 % create the stiffness matrix
                 for i0=1:3
                    i = conn(i0, l);
                    for j0=1:3
                        j = conn(j0,l);                
                        stiff(i,j)  =  stiff(i,j) + G(i0,j0)*Ar/3.0;
                    end

                 end 
                
                 % create the mass matrix
                 mass(conn(:,l)',conn(:,l)') = mass(conn(:,l)',conn(:,l)') + Ar * [ 2, 1, 1; 1, 2, 1; 1, 1, 2 ] / 24;
                
                
            end
            
            % create the initial condition for the solution
            U(:,1) = initiate( coor(1,:), coor(2,:), 0, handles );
 
        
        for t=2:ceil(T/dt)+1
            % Take out domain number from progress bar caption.  JPW
            % progressbar(t/(T/dt+1),0,['Solving Domain #',num2str(k)]);
            progressbar(t/(T/dt+1), 0, 'Solution progress');
            
            b = sparse(ndf,1);  
            
             
             u = sparse(ndf,1); 
             
             for l=1:size(curDomain.mesh.conn,2)

%                 curElement = curDomain.mesh.conn(l);

                % get the coordinates of the 3 nodes of the element
                %
                % xa ya
                % xb yb
                % xc yc

                c = coor(:,conn(:,l))';

                % center of the triangle
                % checked
                center = sum(c)/3;


                % triangle area
                Ar = abs((c(2,1)*c(1,2)-c(1,1)*c(2,2))+(c(3,1)*c(2,2)-c(2,1)*c(3,2))+(c(1,1)*c(3,2)-c(3,1)*c(1,2)))/2;

                % transform to
                %
                % 1 xa ya
                % 1 xb yb
                % 1 xc yc        
                M = [ones(3,1),c];

                % solve z=g(1)+g(2)x+g(3)y       
                g = inv(M);

                % compute grad phi1 . grad phi 2
                G = g(2:3,:)'*g(2:3,:);


                % value in the center
                fv = fFn(center(1), center(2), t*dt,handles );
                   

   

                b(conn(:,l)) = b(conn(:,l)) + Ar * dt *fv/6;

             end

            b =  b + mass*U(:,t-1);
            
            % this is the point at which the dirichlet boundary conditions are
            % defined.
            u(diricVertices) = u(diricVertices)+diricFn( coor(1,diricVertices), coor(2,diricVertices), t*dt,types, conditions, diricVertices)';
            b = b - (dt*stiff + mass)* u;
            
            % implement the neumann conditions
            b(neumannVertices) = b(neumannVertices) + neumannFn(coor(1,:), coor(2,:), t*dt,dt,ggedges(4,:), conditions, neumannVertices, neumannEdges);


            % Solving the system with NAG routines f11
            % the system is A * x = bp
            tic;
                A = tril(dt*stiff(freeVertices, freeVertices) + mass(freeVertices, freeVertices));
                bp = b(freeVertices);

                
                % get the matrix in the symetric co-ordinate storage format
                [irow, icol, a] = find(A);
                
                % size of the system
                n = nag_int(length(freeVertices));
                nnz = nag_int(length(a));

                % call the reordering routine for the columns and rows
                % index
                [nnz, a, irow, icol, istr, ifail] = f11zb(n, nnz, a, nag_int(irow), nag_int(icol), 'R','R');

                % parameters for the factorization
                lfill = nag_int(0);
                dtol = 0;
                mic = 'N';
                dscale = 0;


                
                ipiv = nag_int(zeros(nnz,1));
                la = nag_int(7*double(nnz));

                % dirty vector expansion to allocate memory    
                tz = zeros(la,1);

                tz(1:double(nnz),1) = irow;
                irow = tz;

                tz(1:double(nnz),1) = icol;        
                icol = tz;

                tz(1:double(nnz),1) = a;
                a = tz;        

                % factorization
                [aOut, irowOut, icolOut, ipivOut, istr, nnzc, npivm, ifail] = f11ja(nnz, a, nag_int(irow), nag_int(icol), lfill, dtol, mic, dscale, ipiv,'la',la,'n',n);

                % solving parameters
                tol = 1e-06;
                maxitn = nag_int(100);

                x = zeros(double(nnz),1);
                
                % another dirty vector expansion
                tz = zeros(la,1);            
                tz(1:double(n),1) = bp;
                bp =  tz;
                
                % solving 
                [xOut, rnorm, itn, ifail] = f11jc('CG', nnz, aOut, irowOut, icolOut, ipivOut, istr, bp, tol, maxitn, x,'n',n);

                u(freeVertices) = xOut(1:double(n),:);
            tn = toc;
            
%            tic    
%            % Solving the system with matlab \ operator
%            u(freeVertices) = (dt*stiff(freeVertices, freeVertices) + mass(freeVertices, freeVertices))\ b(freeVertices);
%            tm = toc;
            
%            fprintf('Time with NAG: %f with Matlab: %f %f\n',tn,tm,tn/tm);
            
%             kk = (dt*stiff(freeVertices, freeVertices) + mass(freeVertices, freeVertices))\ b(freeVertices);
%             sum((u(freeVertices)-kk).^2)
            
%             sol = full(u);
            U(:,t) = u;


%             stats = [min(sol);max(sol);mean(sol)];
%             fprintf('\nMin:\t%.3f\nMax:\t%.3f\nMean:\t%.3f\n', stats );

        end
        handles.fem.solutions(k).result = U;
    end
    
    res = handles;


end

function res = fFn(x,y, t,handles)
    switch handles.fem.equation
        case 'poisson'
            res = eval(handles.fem.poisson.source);
        case 'laplace'
            res = 0;
        case 'maxwell'
            res = -1;
            
        
    end
end

% diricFn evaluates the boundary conditions on the constrained edges
function res = diricFn(xx,yy, t,ttype, conditions, diricVerts)
    
    if ~isempty(diricVerts)    
        nb = size(diricVerts,2);
    else
        nb = 0;
    end
    
    res = zeros(1,nb);
 
    for i=1:double(nb)
        type = ttype(i);
        x = xx(i);
        y = yy(i);
        res(i) = eval(conditions(type).expression, 0 );
    end

end

% works out the neumann boundary condition
function res = neumannFn(xx, yy, t,dt,ttype, conditions, neumannVerts, neumannEdges)

    
    nb = size(xx,2);
    
    sol = zeros(1,nb);
    
    for i = 1:size(neumannVerts,2)-1
        edge = neumannEdges(:,i);
        % find vertices associated with this edge, probably edge(1) and
        % edge(2)
        p = edge(1);
        q = edge(2);
        
        length = norm([xx(p);yy(p)] - [xx(q);yy(q)]);
        
        type = edge(4);
        
        % put the mid point of the edge into x and y
        x = (xx(p) + xx(q))/2;
        y = (yy(p) + yy(q))/2;
        
        sol(p) = sol(p) + eval(conditions(type).expression,0)*length*dt/2;
        sol(q) = sol(q) + eval(conditions(type).expression,0)*length*dt/2;
        
    end
    
    res = sol(neumannVerts)';

end

function u = initiate( x, y, t, handles )

    u = eval(handles.fem.init,'0')';


end

